@@ -10,7 +10,7 @@ from PIL import Image
 from IPython.core.display import display, HTML
 from graphviz import Digraph
 
-re_queque = re.compile(r"(Z\d+|epsilon|_[^_]+_|\w)")
+re_queque = re.compile(r"(Z_?\d+|Z₀|epsilon|_[^_]+_|\w)")
 
 
 class PushDownAutomaton:
@@ -298,9 +298,11 @@ class PushDownAutomaton:
         :param z: Stack symbol
         :returns: Destination state or states"""
         qs = self._get_transition(self._nstate(q), self._nsymbol(a), self._nqsymbol(z))
-        return [(self._state(q), tuple([self._qsymbol(z) for z in s])) for q, s in qs]
+        return set(
+            (self._state(q), tuple([self._qsymbol(z) for z in s])) for q, s in qs
+        )
 
-    def add_transition(self, q_i, a, z, qs, force=False):
+    def add_transition(self, q_i, a, z, qs, force=False, update=False):
         """Adds a transition
 
         :param q_i: Source state
@@ -337,7 +339,8 @@ class PushDownAutomaton:
         ]
         try:
             self.ttable[nq_i][na][nz]
-            raise AlreadyExistsPDATransition(q_i, a, z, self)
+            if not update:
+                raise AlreadyExistsPDATransition(q_i, a, z, self)
         except KeyError:
             pass
 
@@ -384,9 +387,9 @@ class PushDownAutomaton:
 
         :param a: Symbol
         :returns: Indixes of symbol"""
-        if a in self.gamma:
-            raise AlreadyExistsSymbol(self._filter(a))
-        self.gamma.add(a)
+        if a in self.sigma:
+            raise AlreadyExistsSymbol(a)
+        self.sigma.add(a)
         return self.sigma.add(a)
 
     def add_qsymbol(self, z):
@@ -450,7 +453,10 @@ class PushDownAutomaton:
 
         :param w: String
         :returns: None"""
-        return self.acceptor(set([q for q, _ in self.delta_extended(None, w).keys()]))
+        try:
+            return self.acceptor(set([q for q, _ in self.delta_extended(None, w).keys()]))
+        except KeyError:
+            return False
 
     def acceptor(self, q):
         """Checks if state is an acceptor state
@@ -514,6 +520,84 @@ class PushDownAutomaton:
         ]
         return "\n".join(info)
 
+    def to_string(self, order_Q=None, order_sigma=None, order_gamma=None):
+        """Creates a string
+
+        :param order_Q: Order to print states
+        :param order_sigma: Order to print alphabet
+        :param order_gamma: Order to print stack alphabet
+
+        :returns: None
+        """
+        largest_q = max([len(q) for q in self.Q])
+        largest_q += 3
+        largest_cell = {a: 2 for a in self.sigma}
+        ttable = defaultdict(list)
+        ttable_ = defaultdict(list)
+        for (q_i, a, z), (q_f, r) in self.items():
+            ttable[(q_i, a, z)].append(f"{z}/{''.join(r)}→{q_f}")
+            ttable_[(q_i, a)].append(ttable[(q_i, a, z)][-1])
+        for (q_i, a), elements_cell in ttable_.items():
+            largest_cell[a] = max(largest_cell[a], len(",".join(elements_cell)))
+        order_Q = list(self.Q) if not order_Q else order_Q
+        order_sigma = list(self.sigma) if not order_sigma else order_sigma
+        order_gamma = list(self.gamma) if not order_gamma else order_gamma
+        strings = []
+        # Alphabet line
+        strings.append(
+            " " * (largest_q + 2)
+            + "|"
+            + "|".join(
+                " {: ^{width}} ".format(a, width=l)
+                for a in order_sigma
+                if (l := largest_cell[a])
+            )
+            + "|"
+        )
+        # Printing states
+        for q_i in order_Q:
+            q_l = q_i
+            if q_i == self.q_0:
+                q_l = f"->{q_i}"
+            if q_i in self.A:
+                q_l = f"{q_l}]"
+            if q_i == self.q_0:
+                strings.append(" {: <{width}}|".format(q_l, width=largest_q + 1))
+            else:
+                strings.append("   {: <{width}}|".format(q_l, width=largest_q - 1))
+            line = []
+            for a in order_sigma:
+                s = []
+                for z in order_gamma:
+                    e = ttable[(q_i, a, z)]
+                    if len(e) > 0:
+                        s.append(",".join(e))
+                line.append(",".join(s))
+            r = "|".join(
+                " {: ^{width}} ".format(c, width=l)
+                for c, a in zip(line, order_sigma)
+                if (l := largest_cell[a])
+            )
+            strings[-1] += r + "|"
+        return "\n".join(strings)
+
+    def save_file(
+        self, filename="machine.txt", order_Q=None, order_sigma=None, order_gamma=None
+    ):
+        """Saves a file
+
+        :param filename: Name of filename (default is machine.txt)
+        :param order_sigma: Order to print alphabet
+        :param order_gamma: Order to print stack alphabet
+
+        :returns: None
+        """
+        string = self.to_string(
+            order_Q=order_Q, order_sigma=order_sigma, order_gamma=order_gamma
+        )
+        with open(filename, "w") as f:
+            f.write(string)
+
     def print_summary(self):
         """Prints a summary of the PDA"""
         print(self.summary())
