@@ -181,8 +181,10 @@ class TuringMachine:
                     tp[c] = t_
                 c = c + Dir
                 if c < 0 and len(tn) <= abs(c):
+                    tn = list(tn)
                     tn.append(self._ntsymbol(self.B))
                 if c >= 0 and len(tp) <= abs(c):
+                    tp = list(tp)
                     tp.append(self._ntsymbol(self.B))
                 states_.add(
                     (nq_f, tuple(tn), tuple(tp), (len(tn) + c, self._tsymbol(na)))
@@ -304,7 +306,7 @@ class TuringMachine:
         qs = self._get_transition(self._nstate(q), self._nsymbol(a))
         return [(self._state(s), self.tsymbol(a), d) for s, a, d in qs]
 
-    def add_transition(self, q_i, t, qs, force=False):
+    def add_transition(self, q_i, t, qs, force=False, update=False):
         """Adds a transition
 
         :param q_i: Source state
@@ -320,16 +322,33 @@ class TuringMachine:
             nt = self.add_tsymbol(t)
         except AlreadyExistsSymbol:
             nt = self._ntsymbol(t)
+
+        #TODO: Differenciate among sigma and gamma
+
+        if force:
+            for q_f, a_, _ in qs:
+                try:
+                    self.add_state(q_f)
+                except AlreadyExistsState:
+                    pass
+                try:
+                    self.add_tsymbol(a_)
+                except AlreadyExistsSymbol:
+                    pass
+
         qs = [(self._nstate(q), self._ntsymbol(t), Dir) for q, t, Dir in qs]
 
         if nq_i in self.ttable and nt in self.ttable[nq_i]:
-            raise AlreadyExistsTMTransition(q_i, t, self)
-
-        if not nq_i in self.ttable:
-            self.ttable[nq_i] = {}
-        if not nt in self.ttable[nq_i]:
-            self.ttable[nq_i][nt] = set()
-        self.ttable[nq_i][nt].update(qs)
+            if update:
+                self.ttable[nq_i][nt].update(qs)
+            else:
+                raise AlreadyExistsTMTransition(q_i, t, self)
+        else:
+            if not nq_i in self.ttable:
+                self.ttable[nq_i] = {}
+            if not nt in self.ttable[nq_i]:
+                self.ttable[nq_i][nt] = set()
+            self.ttable[nq_i][nt].update(qs)
 
     def add_state(self, q, initial=False):
         """Adds a state
@@ -506,6 +525,78 @@ class TuringMachine:
             )
         return "|".join(res)
 
+    def to_string(self, order_Q=None, order_gamma=None):
+        """Creates a string
+
+        :param order_Q: Order to print states
+        :param order_sigma: Order to print alphabet
+        :param order_gamma: Order to print stack alphabet
+
+        :returns: None
+        """
+        largest_q = max([len(q) for q in self.Q])
+        largest_q += 3
+        largest_cell = {a: 2 for a in self.gamma}
+        ttable = defaultdict(list)
+        ttable_ = defaultdict(list)
+        for (q_i, a), (q_f, a_, Dir) in self.items():
+            ttable[(q_i, a)].append(f"{a}/{a_}→{q_f}:{self._dir(Dir)}")
+        for (q_i, a), elements_cell in ttable.items():
+            largest_cell[a] = max(largest_cell[a], len(",".join(elements_cell)))
+        order_Q = list(self.Q) if not order_Q else order_Q
+        order_gamma = list(self.gamma) if not order_gamma else order_gamma
+        strings = []
+        # Alphabet line
+        strings.append(
+            " " * (largest_q + 2)
+            + "|"
+            + "|".join(
+                " {: ^{width}} ".format(a, width=l)
+                for a in order_gamma
+                if (l := largest_cell[a])
+            )
+            + "|"
+        )
+        # Printing states
+        for q_i in order_Q:
+            q_l = q_i
+            if q_i == self.q_0:
+                q_l = f"->{q_i}"
+            if q_i in self.A:
+                q_l = f"{q_l}]"
+            if q_i == self.q_0:
+                strings.append(" {: <{width}}|".format(q_l, width=largest_q + 1))
+            else:
+                strings.append("   {: <{width}}|".format(q_l, width=largest_q - 1))
+            line = []
+            for a in order_gamma:
+                e = ttable[(q_i, a)]
+                line.append(",".join(e))
+            r = "|".join(
+                " {: ^{width}} ".format(c, width=l)
+                for c, a in zip(line, order_gamma)
+                if (l := largest_cell[a])
+            )
+            strings[-1] += r + "|"
+        return "\n".join(strings)
+
+    def save_file(
+        self, filename="machine.txt", order_Q=None, order_gamma=None
+    ):
+        """Saves a file
+
+        :param filename: Name of filename (default is machine.txt)
+        :param order_gamma: Order to print stack alphabet
+
+        :returns: None
+        """
+        string = self.to_string(
+            order_Q=order_Q, order_gamma=order_gamma
+        )
+        with open(filename, "w") as f:
+            f.write(string)
+
+
     def summary(self):
         """Producrs summary of the PDA
         :returns: List with summary"""
@@ -517,7 +608,7 @@ class TuringMachine:
             "Aceptors: " + ", ".join(self.A),
             "Transitions:\n"
             + "\n".join(
-                f" {q_i},{t}/{t_} → {q_f},{Dir}"
+                f" {q_i},{t}/{t_} → {q_f}:{self._dir(Dir)}"
                 for (q_i, t), (q_f, t_, Dir) in self.items()
             ),
         ]
