@@ -179,7 +179,7 @@ class FiniteAutomaton:
         try:
             na = self._nsymbol(a)
         except KeyError:
-            if force and not a is "ε":
+            if force and a != "ε":
                 na = self.add_symbol(a)
             else:
                 raise DoesNotExistsSymbol(a)
@@ -561,11 +561,14 @@ class FiniteAutomaton:
             order_Q = list(self.Q)
         if not order_sigma:
             order_sigma = list(self.sigma)
-        strings=[]
+        strings = []
         # Alphabet line
         strings.append(
-                    " " * (largest_q + 2)+"|"+"|".join(
-                        "{: ^{width}}".format(a, width=largest_cell) for a in self.sigma)+"|")
+            " " * (largest_q + 2)
+            + "|"
+            + "|".join("{: ^{width}}".format(a, width=largest_cell) for a in order_sigma)
+            + "|"
+        )
         # Printing states
         for q_i in order_Q:
             q_l = q_i
@@ -574,26 +577,23 @@ class FiniteAutomaton:
             if q_i in self.A:
                 q_l = f"{q_l}]"
             if q_i == self.q_0:
-                strings.append(
-                    " {: <{width}}|".format(q_l, width=largest_q + 1))
+                strings.append(" {: <{width}}|".format(q_l, width=largest_q + 1))
             else:
-                strings.append(
-                    "   {: <{width}}|".format(q_l, width=largest_q - 1))
+                strings.append("   {: <{width}}|".format(q_l, width=largest_q - 1))
             line = []
             for a in order_sigma:
                 try:
-                    s=self[(q_i, a)]
-                    if isinstance(s,set):
+                    s = self[(q_i, a)]
+                    if isinstance(s, set):
                         line.append(",".join(s))
                     else:
                         line.append(s)
                 except DoesNotExistsTransition:
                     line.append("")
             r = "|".join("{: ^{width}}".format(c, width=largest_cell) for c in line)
-            strings[-1]+=r+"|"
+            strings[-1] += r + "|"
         return "\n".join(strings)
 
-
     def save_file(self, filename="machine.txt", order_Q=None, order_sigma=None):
         """Saves a file
 
@@ -603,7 +603,7 @@ class FiniteAutomaton:
 
         :returns: None
         """
-        string=self.to_string(order_Q=order_Q,order_sigma=order_sigma)
+        string = self.to_string(order_Q=order_Q, order_sigma=order_sigma)
         with open(filename, "w") as f:
             f.write(string)
 
